import 'package:flutter/material.dart';
import '../marker_handler/marker_data.dart';
import '../consts.dart';
import 'quick_view_chart.dart';
import 'stat_charts.dart';
import 'sat_layer.dart';
import 'package:flutter_map/flutter_map.dart';
import 'package:latlong2/latlong.dart';


/// MapContainerWidget is the main widget that contains the map with all
/// its layers, polygons and markers.
class MapContainerWidget extends StatefulWidget {
  final List<Measurement> markerList;

  const MapContainerWidget({Key? key, required this.markerList}) : super(key: key);

  @override
  _MapContainerWidgetState createState() => _MapContainerWidgetState();
}

class _MapContainerWidgetState extends State<MapContainerWidget> {

  Measurement? selectedMarker;  // Containing data for selected marker
  bool isMinimized = true;      // Quick view box state tacker
  bool satLayer = false;        // Satellite layer visibility tracker
  bool isTapped = false;        // Button tap state tracker
  final MapController _mapController = MapController(); // Map controller to re-center map view


  // recenterMap moves the map back to its initial view
  void recenterMap() {
    _mapController.move(mapCenter, 9.0);
  }

  @override
  Widget build(BuildContext context) {
    // Initialise selectedMarker to first element in markerList
    selectedMarker ??= widget.markerList[0];

    const double contPadding = 30; // Container padding space

    return LayoutBuilder(
      builder: (context, constraints) {
        double screenWidth = constraints.maxWidth;
        double boxWidth = 0.86;
        double boxHeight = 1.4;
        return Column(
          children: [
            const SizedBox(height: contPadding),
            /*if (true) NB: add search bar
              const SearchBar(),
              const SizedBox(height: contPadding),*/
            ClipRRect(
              borderRadius: BorderRadius.circular(20),
              child: Stack( // Stack of quick view, map layer, satellite layer, and buttons
                children: [
                  SizedBox(
                    width: screenWidth * boxWidth,
                    height: screenWidth * boxHeight,
                    child: FlutterMap(
                      options: MapOptions(
                        center: mapCenter,
                        zoom: 9.0,
                      ),
                      mapController: _mapController,
                      children: [
                        TileLayer(
                          urlTemplate: "https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png",
                          subdomains: const ['a', 'b', 'c'],
                        ),
                        PolygonLayer(
                          polygons: widget.markerList.map((Measurement measurement) {
                            // Map corners to a list of LatLng objects
                            List<LatLng> points = measurement.cornerList.map((Corner corner) {
                              return LatLng(corner.latitude, corner.longitude);
                            }).toList();

                            return Polygon(
                              points: points, // Use list of corner coordinates to render polygon
                              color: Colors.blue.withOpacity(0.5),
                              isFilled: true,
                            );
                          }).toList(),
                        ),
                        MarkerLayer(
                          markers: widget.markerList.map((Measurement measurement) {

                            List<LatLng> corners = measurement.cornerList.map((Corner corner) {
                              return LatLng(corner.latitude, corner.longitude);
                            }).toList();

                            // point calculates the middle point between corners
                            LatLng point(List<LatLng> coordinates) {
                              double averageLatitude = 0.0;
                              double averageLongitude = 0.0;

                              for (LatLng point in coordinates) {
                                averageLatitude += point.latitude;
                                averageLongitude += point.longitude;
                              }

                              // Calculate average latitude and longitude
                              averageLatitude /= coordinates.length;
                              averageLongitude /= coordinates.length;

                              return LatLng(averageLatitude, averageLongitude); // Return the middle point
                            }

                            return Marker(
                              width: 50,
                              height: 50,
                              point: point(corners),
                              builder: (ctx) => GestureDetector(
                                onTap: () {
                                  setState(() {
                                    selectedMarker = measurement;
                                  });
                                },
                                child: const Icon(
                                  Icons.severe_cold,
                                  color: Colors.blue,
                                  size: 30.0,
                                ),
                              ),
                            );
                          }).toList(),
                        ),

                      ],
                    ),
                  ),
                  /*SizedBox(
                    width: screenWidth * boxWidth,
                    height: screenWidth * boxHeight,
                    child: Stack(
                      children: [
                        SatLayer(markerList: widget.markerList), // Satellite layer
                        Visibility(
                          visible: satLayer, // Only show layer if satellite button is toggled on
                          child: SatLayer(markerList: widget.markerList),
                        ),
                      ],
                    ),
                  ),*/
                  Positioned( // Quick view box layered over map
                    bottom: 10,
                    right: 10,
                    child: ClipRRect(
                      borderRadius: BorderRadius.circular(10),
                      child: Container(
                        width: (screenWidth * boxWidth) / 2.4,
                        height: isMinimized ? 20 : (screenWidth * boxWidth) / 2.4,
                        color: Colors.blue.withOpacity(0.7),
                        child: Stack(
                          children: [
                            Visibility( // Graph only visible when box is maximized and a marker is selected
                              visible: !isMinimized && selectedMarker != null,
                              child: Center(
                                child: Padding(
                                  padding: const EdgeInsets.only(right: 16.0, top: 17.0),
                                  child: SizedBox(
                                    width: (screenWidth * boxWidth) / 2.3,
                                    height: (screenWidth * boxWidth) / 2.3,
                                    child: const QuickViewChart(), // Quick view graph
                                  ),
                                ),
                              ),
                            ),
                            Positioned(
                              top: 0,
                              right: 5,
                              child: GestureDetector(
                                onTap: () {
                                  setState(() {
                                    isMinimized = !isMinimized; // Toggle minimized state
                                  });
                                },
                                child: Icon(isMinimized ? Icons.arrow_drop_up : Icons.arrow_drop_down),
                              ),
                            ),
                          ],
                        ),
                      ),
                    ),
                  ),
                  Positioned( // Satellite button
                    top: 10,
                    right: 10,
                    child: GestureDetector(
                      onTap: () {
                        setState(() {
                          satLayer = !satLayer; // Toggle satellite layer state on press
                        });
                      },
                      child: Container(
                        padding: const EdgeInsets.all(8),
                        decoration: satLayer ? const BoxDecoration( // Add decoration only when pressed
                          shape: BoxShape.circle,
                          color: Colors.blue,
                        ) : null,
                        child: const Icon(Icons.satellite_alt_outlined),
                      ),
                    ),
                  ),
                  Positioned( // Back to center button
                    top: 45,
                    right: 10,
                    child: GestureDetector(
                      onTapDown: (_) {
                        setState(() {
                          recenterMap(); // Reset map view
                          isTapped = true;
                        });
                      },
                      onTapUp: (_) {
                        setState(() {
                          isTapped = false;
                        });
                      },
                      onTapCancel: () {
                        setState(() {
                          isTapped = false;
                        });
                      },
                      child: Container(
                        padding: const EdgeInsets.all(8),
                        decoration: isTapped ? const BoxDecoration( // Add decoration only when pressed
                          shape: BoxShape.circle,
                          color: Colors.blue,
                        ) : null,
                        child: const Icon(Icons.settings_backup_restore),
                      ),
                    ),
                  ),
                ],
              ),
            ),
            const SizedBox(height: contPadding), // Padding between containers
            ClipRRect(
              borderRadius: BorderRadius.circular(20),
              child: SizedBox(
                width: screenWidth * boxWidth,
                height: screenWidth * boxHeight * 1.5, // NB: make dynamic
                child: Align(
                  alignment: Alignment.topLeft,
                  child: Padding(
                    padding: const EdgeInsets.only(top: 20, left: 20), // Edge padding, text
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                         Text(
                          'Ice stats',
                          style: titleStyle,
                        ),
                        const Divider(),
                        Text(
                          'Time of measurement',
                          style: subHeadingStyle,
                        ),
                        Text(
                          'Date ${(selectedMarker?.timeMeasured.day ?? '-')}/${(selectedMarker?.timeMeasured.month ?? '-')}/${(selectedMarker?.timeMeasured.year ?? '-')}',
                          style: regTextStyle,
                        ),
                        Text(
                          'Time: ${selectedMarker?.timeMeasured.hour}:00',
                          style: regTextStyle,
                        ),
                        const SizedBox(height: contPadding),
                        Text(
                          'Measuring point: (${selectedMarker?.dataList[0].latitude}, ${selectedMarker?.dataList[0].latitude})',
                          style: regTextStyle,
                        ),
                        const SizedBox(height: contPadding),
                        const StatCharts(),
                      ],
                    ),
                  ),
                ),
              ),
            ),
          ],
        );
      },
    );
  }
}